#include "stdafx.h"
#undef min
#undef max
#include "Enemy.h"
#include "TextureManager.h"
#include "Hero.h"
#include "Level.h"
#include "resource.h"
#include "EnemyMgr.h"

namespace LD46
{
	long Enemy::s_id = 1;

	Enemy::Enemy(TextureManager& txMgr, EnemyType et,float radius, EnemyMgr* mgrPtr)
		:m_txMgr(txMgr)
		,m_radius(radius)
		,m_pwr(1.0)
		,m_ticker(0.0)
		,m_enemyType(et)
		,m_hitTick(0.0)
		,m_victoryTick(0.0)
		,m_toggle(false)
		,m_id(s_id++)
		, m_firstRound(true)
		, m_enemyMgrPtr(mgrPtr)
	{		
		switch (m_enemyType)
		{
		case EnemyType::Gargoyle:
		case EnemyType::Bat:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_BAT, 64, 32, 0.8f, false);
			m_spr->SetOrigin(sf::Vector2f(32,32));
			break;
		case EnemyType::Blob:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_BLOB, 32, 32, 0.5f, false);
			m_spr->SetOrigin(sf::Vector2f(16, 0));
			break;
		case EnemyType::Demon:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_DEVIL, 128, 128, 0.5f, false);
			m_spr->SetOrigin(sf::Vector2f(64, 64));
			break;
		case EnemyType::Boss1:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_BOSS1, 128, 128, 0.5f, false);
			m_spr->SetOrigin(sf::Vector2f(32, 64));
			break;
		case EnemyType::Boss3:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_DEMONHEAD, 256, 128, 0.2f, false);
			m_spr->SetOrigin(sf::Vector2f(128, 110));
			break;
		case EnemyType::Boss2:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_BLOB, 32, 32, 0.5f, false);
			m_spr->SetOrigin(sf::Vector2f(16, 0));
			for (int i = 0; i != 64; ++i)
			{
				auto blb = std::make_unique<AnimatedSprite>(txMgr, IDB_BLOB, 32, 32, 0.5f, false);
				blb->SetOrigin(sf::Vector2f(16, 0));
				blb->SetScale(sf::Vector2f(2.0f, 2.0f));
				m_blobbers.emplace_back(blb.release());
			}
			break;
		case EnemyType::Tentacle:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_TENTACLE, 64, 64, 1.5f, false);
			m_spr->SetOrigin(sf::Vector2f(16, 16));
			break;
		case EnemyType::Tusk:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_TUSK, 64, 64, 1.5f, false);
			m_spr->SetOrigin(sf::Vector2f(32, 32));
			break;
		case EnemyType::Bug:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_BUG, 32, 32, 1.5f, false);
			m_spr->SetOrigin(sf::Vector2f(16, 16));
			break;
		case EnemyType::Mystic:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_MYSTIC, 320, 205, 0.8f, false);
			m_spr->SetOrigin(sf::Vector2f(160.0f, 110.5f));
			break;
		default:
			m_spr = std::make_unique<AnimatedSprite>(txMgr, IDB_BAT, 32, 32, 0.8f, false);
			break;
		}
		m_spr->SetScale(sf::Vector2f(2.0f, 2.0f));
		m_spr->Play();
	}

	Enemy::~Enemy()
	{
	}

	bool Enemy::IsCollision(const sf::Vector3f& pos) const
	{
		auto dx = pos.x - m_pos.x;
		auto dy = pos.y - m_pos.y;
		auto dz = pos.z - m_pos.z;
		auto dd = sqrtf(dx * dx + dy * dy + dz * dz);
		return dd < Radius();
	}

	const sf::Vector3f Enemy::Dir() const
	{
		return m_dir;
	}


	void Enemy::HitByPlayer(const sf::Vector3f& hitDir)
	{
		if (m_hitTick == 0.0)
		{
			m_hitTick = 5.0;
			m_hitByPlayerDir = hitDir*10.0f;
			m_storedPos = m_pos;
		}
	}

	void Enemy::VictoryPause()
	{
		m_victoryTick = 10.0;
	}

	float Enemy::Power() const
	{
		return m_pwr;
	}

	void Enemy::AdjustPower(const float pwr)
	{
		m_pwr += pwr;
	}

	float Enemy::Radius() const
	{
		return m_radius;
	}

	bool Enemy::IsAttacking() const
	{
		return true;
	}

	sf::FloatRect Enemy::Bounds() const
	{
		return sf::FloatRect();// m_spr[0].getGlobalBounds();
	}

	void Enemy::UpdatePosition(const sf::Vector3f& pos, Level& level)
	{
		if (pos == m_pos)
			return;

		//auto enemyOnMap = level.EnemyPos(m_pos);
		//if (enemyOnMap)
		//{
		//	if (enemyOnMap->Id() == Id())
		//		level.ClearEnemy(m_pos);
		//}
		m_pos = pos;
		level.SetEnemy(pos.x / 32, pos.z / 32, shared_from_this());
	}

	bool Enemy::Update(const sf::Time& tm, Hero& hero, Level& level)
	{
		auto& p = Position();
		int qx = p.x / 32;
		int qy = p.z / 32;
		std::set<std::pair<int, int>> usedPos;
		usedPos.insert(std::make_pair(qx, qy));
		if (m_firstRound)
		{
			m_firstRound = false;
			level.SetEnemy(qx , qy, shared_from_this());
			//Creat tentacles here
			if (m_enemyType == EnemyType::Boss1 && m_enemyMgrPtr != nullptr)
			{
				for (int r = 1.0; r < 5.0; r += 1.0)
				{
					for (auto a = 0.0; a < 3.13 * 2; a += 0.1)
					{
						auto tp = p;
						tp.x += r*16.0 * cos(a);
						tp.z += r*16.0 * sin(a);
						tp.y = 8;
						int ex = tp.x / 32;
						int ey = tp.z / 32;
						auto pp = std::make_pair(ex, ey);
						if (usedPos.find(pp) != std::end(usedPos))
							continue;
/*						if (ex == qx && ey == qy)
							continue;
							*/
						usedPos.insert(pp);
						m_tentacles.push_back(m_enemyMgrPtr->CreateTentacle((tp)));
					}
				}
			}
		}
		m_spr->Update(tm);
		switch (m_enemyType)
		{
		case EnemyType::Boss1:
		case EnemyType::Boss2:
		case EnemyType::Boss3:
		case EnemyType::Boss4:
		{
			//UpdatePosition(m_pos, level);
			auto dx = hero.Position().x - p.x;
			auto dz = hero.Position().y - p.z;
			auto dd = sqrtf(dx * dx + dz * dz);
			if (dd > 100)
			{
				level.SetEnemy(m_pos.x / 32, m_pos.z / 32, shared_from_this());
				return true;
			}
		}
			break;
		case EnemyType::Tusk:
			level.SetEnemy(m_pos.x / 32, m_pos.z / 32, shared_from_this());
			return true;
			break;
		default:
			break;
		}
		switch (m_enemyType)
		{
		case EnemyType::Boss1:
			//Creat tentacles here
			if (m_enemyMgrPtr != nullptr)
			{
				auto tcks = 0.05*cos(static_cast<float>((tm).asMilliseconds()) / 200.0);
				auto tock = static_cast<float>((tm).asMilliseconds()) / 200.0;
				int t = 0;
				for (int r = 1.0; r < 5.0; r += 1.0)
				{
					for (auto a = 0.0; a < 3.13 * 2; a += 0.1)
					{
						auto tp = p;
						tp.x += r * 16.0 * cos(a+ tcks);
						tp.z += r * 16.0 * sin(a+ tcks);
						tp.y = 8.0 + r*cos(3.0 * a+ tock);
						int ex = tp.x / 32;
						int ey = tp.z / 32;
						auto pp = std::make_pair(ex, ey);

						//if (ex == qx && ey == qy)
						//	continue;
						if (usedPos.find(pp) != std::end(usedPos))
							continue;

						usedPos.insert(pp);

						if (t < m_tentacles.size())
							m_tentacles[t++]->SetPosition(tp);
						//m_tentacles.push_back(m_enemyMgrPtr->CreateTentacle((tp)));
					}
				}
				UpdatePosition(m_pos, level);
			}
			break;
		case EnemyType::Tentacle:
			break;
		case EnemyType::Boss3:
		{
			if ((tm - m_lastDemonGen).asMilliseconds() > 1000)
			{
				if (m_enemyMgrPtr != nullptr)
				{
					sf::Vector3f dd = p;
					auto rr = hero.Rotation();
					dd.x = p.x+ 40 * sin(rr);
					dd.z = p.z +40 * cos(rr);
					m_enemyMgrPtr->CreateEnemy(dd, EnemyType::Bat);
					dd.x = p.x+40 * cos(rr);
					dd.z = p.z+40 * sin(rr);
					m_enemyMgrPtr->CreateEnemy(dd, EnemyType::Bug);
				}
				m_lastDemonGen = tm;
			}
			UpdatePosition(m_pos, level);
		}
			break;
		case EnemyType::Boss2:
			if (m_blobbers.size() > 0)
			{
				const auto& sprPos = m_spr->Pos();
				sf::Vector2f bp;
				int cnt = 0;
				auto sc = m_spr->Scale();
				auto tcks = static_cast<float>((tm).asMilliseconds()) / 2000.0;
				for (int yy = 0; yy != 8; ++yy)
				{
					for (int xx = 0; xx != 8; ++xx)
					{
						bp.x = sprPos.x - sc.x * (4 + xx * 8 + cos((3.0f * static_cast<float>(xx + yy) + tcks) * 4) * 16);
						bp.y = sprPos.y - sc.y * (4 + yy * 8 + sin((4.0f * static_cast<float>(xx + yy) + tcks) * 3) * 16);
						m_blobbers[cnt++]->SetPos(bp);
					}
				}
			}
			break;
		default:
			if (m_enemyType != EnemyType::Mystic)
			{
				auto& heroPos = level.HeroPos();
				auto  oldPos = m_pos;
				sf::Vector3f pos = m_pos;
				auto dx = heroPos.x - m_pos.x;
				auto dz = heroPos.z - m_pos.z;
				auto dl = sqrtf(dx * dx + dz * dz);

				auto dd = (dl > 5) ? 1.0 / dl : 0.1 / dl;;
				pos.x += dx * dd;
				pos.z += dz * dd;
				m_dir = pos - oldPos;
				switch (m_enemyType)
				{
				case EnemyType::Bat:
				case EnemyType::Gargoyle:
					pos.y = 10 + 10 * cos(Id() * 0.3f + static_cast<float>((tm).asMilliseconds()) / 200.0f);
					break;
				default:
					break;
				}
				UpdatePosition(pos, level);
			}
		}

		prevTime = tm;
		return m_pwr > 0;
	}

	void Enemy::Draw(sf::RenderWindow& wnd)
	{
		m_ticker += 0.1;
		m_spr->Draw(wnd);
		if (m_blobbers.size() > 0)
		{
			auto sc = m_spr->Scale();
			float bc = 0.0f;
			for (auto& b : m_blobbers)
			{
				auto sx= sc.x*0.9 +cos(m_ticker + bc) * 0.3;
				auto sy = sc.y*0.9 + cos(m_ticker+bc) * 0.3;
				bc += 0.03;
				b->SetScale(sc);
				b->Draw(wnd);
			}
		}
	}

	void Enemy::SetPosition(const sf::Vector3f& pos)
	{
		m_pos = pos;
	}
	const sf::Vector3f & Enemy::Position() const
	{
		return m_pos;
	}

	Enemy::EnemyType Enemy::TheType() const
	{
		return m_enemyType;
	}

	void Enemy::SetSpritePosition(const sf::Vector2f& pos)
	{
		m_spr->SetPos(pos);
	}

	const sf::Vector2f& Enemy::SpritePosition() const
	{
		return m_spr->Pos();
	}

	void Enemy::SetScale(const sf::Vector2f& scale)
	{
		sf::Vector2f sc(scale.x * 0.2f, scale.y * 0.2f);
		m_spr->SetScale(sc);
	}

	const std::vector<Enemy*>& Enemy::Tentacles() const
	{
		return m_tentacles;
	}


	long Enemy::Id() const
	{
		return m_id;
	}
}