#pragma once

#include <SFML/Graphics.hpp>
#include <memory>
#include "AnimatedSprite.h"

namespace LD46
{
	class IShotFactory;
	class TextureManager;
	class Hero;
	class Level;
	class EnemyMgr;

	//There's no time to write all the boilerplate code for a subclass-per-enemy type during a game jam...
	class Enemy : public std::enable_shared_from_this<Enemy>
	{
	public:
		enum EnemyType
		{
			Bat,
			Blob,
			Gargoyle,
			Demon,
			Bug,
			Mystic,
			Boss1,
			Boss2,
			Boss3,
			Boss4,
			Tentacle,
			Tusk,
			COUNT
		};
		Enemy(TextureManager& txMgr, EnemyType et, float radius,EnemyMgr* mgrPtr = nullptr/*Bad Idea but I really dont have time*/);
		virtual ~Enemy();

		bool Update(const sf::Time& tm, Hero& hero, Level& level);
		void Draw(sf::RenderWindow& wnd);
		long Id() const;
		bool IsCollision(const sf::Vector3f& pos) const;
		const sf::Vector3f Dir() const;
		void HitByPlayer(const sf::Vector3f& hitDir);
		void VictoryPause();
		float Power() const;
		void AdjustPower(const float pwr);
		float Radius() const;
		bool IsAttacking() const;
		sf::FloatRect Bounds() const;
		void SetPosition(const sf::Vector3f& pos);
		const sf::Vector3f& Position() const;
		EnemyType TheType() const;
		void SetSpritePosition(const sf::Vector2f& pos);
		const sf::Vector2f& SpritePosition() const;
		void SetScale(const sf::Vector2f& scale);
		const std::vector<Enemy*>& Tentacles() const;
	private:

		void UpdatePosition(const sf::Vector3f& pos, Level& level);

		static long s_id;
		const long m_id;

		void GetTexture(TextureManager& txMgr, Enemy::EnemyType et);

		const float m_radius;
		sf::Time m_timeSinceLastShot;
		TextureManager& m_txMgr;
		EnemyMgr* m_enemyMgrPtr;
		sf::Vector3f m_dir;
		sf::Vector3f m_hitByPlayerDir;

		std::unique_ptr<AnimatedSprite> m_spr;
		sf::Vector3f m_pos;
		sf::Vector3f m_storedPos;
		
		float m_pwr;
		float m_ticker;
		bool m_toggle;
		const EnemyType m_enemyType;
		sf::Time m_lastDemonGen;
		float m_hitTick;
		float m_victoryTick;
		bool m_firstRound;
		sf::Time prevTime;
		std::vector<Enemy*> m_tentacles;
		std::vector<std::unique_ptr<AnimatedSprite>> m_blobbers;
	};

	typedef std::shared_ptr<Enemy> EnemyPtr;
}
