#pragma once
#include "Enemy.h"
#include "Explosion.h"

#include <SFML/Graphics.hpp>
#include <map>
#include <string>
#include <array>
#include <optional>

namespace LD46
{
	class Level;
	class Hero;
	class TextureManager;
	class SoundManager;
	class ParticleMgr;
	class Hud;
	enum class CollisionType;

	class EnemyMgr
	{
	public:
		EnemyMgr(TextureManager& txMgr, SoundManager& soundMgr, ParticleMgr& partMgr, Hud& hud, const int level);
		virtual ~EnemyMgr();
		void Update(const sf::Time& tm, Hero& hero, Level& level);
		bool UpdateShotCollision(const sf::Vector3f& shotPos);
		Enemy* CreateTentacle(sf::Vector3f& pos);
		Enemy* CreateEnemy(const sf::Vector3f& pos, Enemy::EnemyType et);
		void Erase(int id);
		std::optional<sf::Time> MetMystic() const;
	private:
		std::optional<sf::Time> m_metMystic;
		void UpdateEnemies(const sf::Time& tm, LD46::Hero& hero, LD46::Level& level);
		TextureManager& m_txMgr;
		SoundManager& m_sndMgr;
		ParticleMgr& m_partMgr;
		Hud& m_hud;
		const int m_level;
		CollisionType m_prevColl;
		std::map<long, EnemyPtr> m_enemies;
		std::map<long, ExplosionPtr> m_explosions;
		std::array<int/*spwanlimit*/, Enemy::EnemyType::COUNT> m_spawnLimits;
		std::array<sf::Time, Enemy::EnemyType::COUNT> m_lastSpawnTime;
		bool m_bossSpawned[4];
		bool m_firstRound;
		sf::Time m_lastMunch;
		sf::Time m_lastHeroHit;
	};
}