#include "stdafx.h"
#include "Hero.h"
#include "TextureManager.h"
#include "SoundManager.h"
#include "AnimatedSprite.h"

#include "resource.h"

namespace
{
	float sgn(float a)
	{
		return a >= 0 ? 1.0f : -1.0f;
	}
	static bool s_togged = false;
	static int s_togCnt = 0;
}

namespace LD46
{	
	Hero::Hero(TextureManager& txMgr, SoundManager& sndMgr, const float baseLife)
		:m_txMgr(txMgr)
		,m_sndMgr(sndMgr)
		,m_action(Action::None)
		, m_walkTick(0.0)
		,m_currentSprite(0)
		,m_heroSpr(txMgr, IDB_HERORUN, 32, 64, 2.0f, false)
		, m_heroSprStand(txMgr, IDB_HEROSTAND, 32, 64, 2.0f, false)
		, m_heroSprAttack(txMgr, IDB_HEROATTACK, 32, 64, 2.0f, true)
		, m_heroSprHit(txMgr, IDB_HEROHIT2, 32, 64, 0.5f, true)
		,m_rotation(0.0f)
		,m_drawState(State::Standing)
		, m_worldState(WorldState::Darkness)
		, m_justAttacked(false)
		,m_tusk(0)
	{
		for (int i = 0; i != WorldState::COUNT; ++i)
			m_pwr[i] = baseLife;

		m_heroSpr.SetOrigin(sf::Vector2f(0.5f, 0.5f));
		m_heroSpr.SetScale(sf::Vector2f(3.0f, 2.0f));
		m_heroSprStand.SetOrigin(sf::Vector2f(0.5f, 0.5f));
		m_heroSprStand.SetScale(sf::Vector2f(3.0f, 2.0f));
		m_heroSprAttack.SetOrigin(sf::Vector2f(0.5f, 0.5f));
		m_heroSprAttack.SetScale(sf::Vector2f(3.0f, 2.0f));
		m_heroSprHit.SetOrigin(sf::Vector2f(0.5f, 0.5f));
		m_heroSprHit.SetScale(sf::Vector2f(3.0f, 2.0f));

		m_pos = sf::Vector2f(134.0f, 8000.0f);
		//m_fakePos = m_pos;
	}

	bool Hero::Update(const sf::Time& tm)
	{		
		m_lastUpdateTime = tm;
		m_heroSpr.Update(tm);
		m_heroSprStand.Update(tm);
		m_heroSprAttack.Update(tm);
		m_heroSprHit.Update(tm);

		if (m_worldState == WorldState::Chaos)
			m_pwr[WorldState::Chaos] -= 0.004;

		if (fabs(m_knockBack.x) > 0 || fabs(m_knockBack.y) > 0)
		{
			//m_pos += m_knockBack;
			m_knockBack.x *= 0.6;
			m_knockBack.y *= 0.6;
		}

		return true;
	}
	static float ttick = 0.0;
	void Hero::Draw(sf::RenderWindow& wnd)
	{
		switch (m_drawState)
		{
		case State::Running:
			m_heroSpr.Draw(wnd);
			break;
		case State::Attack:
			m_heroSprAttack.Draw(wnd);
			break;
		case State::Hit:
			m_heroSprHit.Draw(wnd);
			break;
		default:
			m_heroSprStand.Draw(wnd);
			break;
		}
	}

	void Hero::KnockBackAdjust(const sf::Vector2f& stepLength)
	{
		m_pos += stepLength;
	}

	void Hero::Walk(const sf::Vector2f& stepLength)
	{	
		if (!m_heroSprAttack.IsFinished())
			return;

		if (!m_heroSprHit.IsFinished())
			return;

		if (stepLength.x == 0 && stepLength.y == 0)
		{
			m_heroSpr.Stop();
			m_heroSprStand.Play();
			m_drawState = State::Standing;
			Act(Action::Standing);
		}
		else
		{
			m_heroSpr.Play();
			m_heroSprStand.Stop();
			m_drawState = State::Running;
			Act(Action::Walk);
		}

		m_stepLength = stepLength;
		m_pos += stepLength;
	}

	void Hero::Act(Action action)
	{
		if (m_drawState == State::Hit)
			return;

		if (action == Action::Attack)
		{
			m_justAttacked = true;
			m_heroSpr.Stop();
			m_heroSprStand.Stop();
			m_heroSprAttack.Play();
			m_drawState = State::Attack;
		}
		m_action = action;
	}

	CollisionType Hero::IsCollision(const sf::Vector3f& pos, const float radius, const sf::FloatRect enemyBounds, const sf::Vector3f& dir, float& dmgScale)
	{
		dmgScale = 1.0;
		return CollisionType::None;
	}

	const sf::Vector2f & Hero::Position() const
	{
		return m_pos;
	}
	void Hero::AdjustPower(const float pwr, const WorldState state)
	{
		auto p = pwr;
		if (pwr < 0 && m_worldState == WorldState::Shadow)
			p *= 3;

		if (pwr < 0)
		{
			m_drawState = State::Hit;
			m_heroSprHit.Play();
			m_heroSpr.Stop();
			m_heroSprStand.Stop();
			m_heroSprAttack.Stop();
		}

		m_pwr[state] += p;
	}
	float Hero::Power(const WorldState state) const
	{
		return m_pwr[state];
	}
	bool Hero::IsAction(Action action) const
	{
		return m_action == action;
	}

	void Hero::ClearAction(Action action)
	{
		m_action = Action::None;
	}

	void Hero::SetSpritePoition(const sf::Vector2f& pos)
	{
		m_heroSpr.SetPos(pos);
		m_heroSprStand.SetPos(pos);
		m_heroSprAttack.SetPos(pos);
		m_heroSprHit.SetPos(pos);
	}

	void Hero::SetRotation(const float rotation)
	{
		m_rotation = rotation;
	}

	const float Hero::Rotation() const
	{
		return m_rotation;
	}

	void Hero::SetWorldState(WorldState state)
	{
		m_worldState = state;
	}

	WorldState Hero::GetWorldState() const
	{
		return m_worldState;
	}

	bool Hero::PerformingHit()
	{
			//::OutputDebugStringA((std::to_string(m_heroSprAttack.CurrentFrame()) + ":" + std::to_string(m_justAttacked) + "\r\n").c_str());
		if (m_justAttacked  && (m_heroSprAttack.IsFinished() || m_heroSprAttack.CurrentFrame() > 3))
		{
			m_justAttacked = false;
			return true;
		}
		return false;
	}


	void Hero::SetKnockBack(const sf::Vector2f& kn)
	{
		m_knockBack = kn;
	}

	const sf::Vector2f& Hero::KnockBack() const
	{
		return m_knockBack;
	}

	void Hero::AddTusk()
	{
		++m_tusk;
	}

	int Hero::Tusks() const
	{
		return m_tusk;
	}


}

