#pragma once
#include <SFML/Graphics.hpp>
#include "AnimatedSprite.h"

namespace LD46
{
	class TextureManager;
	class SoundManager;
	class IConnectable;

	enum class Action
	{
		None,
		Walk,
		Standing,
		Attack
	};

	enum class ObjectType
	{
		None,
		FireSpell,
		WaterSpell,
		StoneSpell,
		ChaosSpell,
		//InvisibleSpell,
		Key,
		//SmartBomb,
		COUNT
	};

	enum class CollisionType
	{
		None,
		PlayerHit,
		EnemyHit,
		Blocked,
		Immortal
	};
	enum WorldState
	{
		Darkness,
		Shadow,
		Chaos,
		COUNT
	};

	class Hero
	{
	public:
		enum class State
		{
			Standing,
			Running,
			Attack,
			Hit
		};
		Hero(TextureManager& txMgr, SoundManager& sndMgr, const float baseLife);
		bool Update(const sf::Time& tm);
		void Draw(sf::RenderWindow& wnd);
		void KnockBackAdjust(const sf::Vector2f& stepLength);
		void Walk(const sf::Vector2f& stepLength);
		void Act(Action action);
		CollisionType IsCollision(const sf::Vector3f& pos, const float radius, const sf::FloatRect enemyBounds, const sf::Vector3f& dir, float& dmgScale);
		const sf::Vector2f& Position() const;
		void AdjustPower(const float pwr, const WorldState state);
		float Power(const WorldState state) const;
		bool IsAction(Action action) const;
		void ClearAction(Action action);
		void SetSpritePoition(const sf::Vector2f& pos);
		void SetRotation(const float rotation);
		const float Rotation() const;
		void SetWorldState(WorldState state);
		WorldState GetWorldState() const;
		bool PerformingHit();
		void SetKnockBack(const sf::Vector2f& kn);
		const sf::Vector2f& KnockBack() const;
		void AddTusk();
		int Tusks() const;
	private:
		TextureManager& m_txMgr;
		SoundManager& m_sndMgr;
		Action m_action;
		double m_walkTick;
		double m_immortalTick;
		sf::Vector2f m_stepLength;
		float m_pwr[WorldState::COUNT];
		//std::vector<int> m_objects;
		AnimatedSprite m_heroSpr;
		AnimatedSprite m_heroSprStand;
		AnimatedSprite m_heroSprAttack;
		AnimatedSprite m_heroSprHit;
		sf::Vector2f m_pos;
		sf::Time m_lastUpdateTime;
		int m_currentSprite;
		float m_rotation;
		State m_drawState;
		WorldState m_worldState;
		bool m_justAttacked;
		sf::Vector2f m_knockBack;
		int m_tusk;
	};
}

