#include "stdafx.h"
#include "Hud.h"
#include "TextureManager.h"
#include "FontTextMgr.h"
#include "Hero.h"
#include "Resource.h"

namespace LD46
{
	Hud::Hud(TextureManager& txMgr, FontTextMgr& ftMgr, HeroPtr hero)
		:m_txMgr(txMgr)
		,m_ftMgr(ftMgr)
		,m_hero(hero)
		,m_countDownTime(0)
		, m_tickIn(-105)
		, m_forceShowText(false)
	{
		
		m_map.setTexture(m_txMgr.GetTexture(IDB_MAP));
		m_map.setPosition(00.0f, 0.0f);
		m_map.setScale(0.25, 0.25);
		m_map.setPosition(576-24, 0);
		m_point.setSize(sf::Vector2f(1.0, 1.0f));
		m_point.setPosition(576 +6, 28);
		m_point.setFillColor(sf::Color(0xff, 0xff, 0xff, 0xff));
		m_point.setOutlineColor(sf::Color(0xff, 0xff, 0xff, 0xff));
		m_map.setColor(sf::Color(0xff, 0xff, 0xff, 0x80));

		m_border.setTexture(m_txMgr.GetTexture(IDB_BORDER));
		m_border.setPosition(00.0f, 0.0f);
		m_overlay.setTexture(m_txMgr.GetTexture(IDB_HUDOVERLAY));
		m_overlay.setPosition(70.0f, 0.0f);

		const float fontY = 10.0f;
		const float iconY = 2.0f;
		auto& fnt = m_ftMgr.LoadFontInResource(IDB_FONT);
		m_infoTxt.setFont(fnt);
		m_infoTxt.setCharacterSize(12);
		m_infoTxt.setFillColor(sf::Color::Yellow);
		m_infoTxt.setPosition(sf::Vector2f(105, 5));
	}
	Hud::~Hud()
	{
	}
	void Hud::Draw(sf::RenderWindow& wnd)
	{
		wnd.draw(m_border);
		if (m_countDownTime > 0 || m_forceShowText)
		{
			if (m_tickIn < 5)
				m_tickIn += 10;

			m_overlay.setPosition(70.0f, m_tickIn - 5);
			m_infoTxt.setPosition(sf::Vector2f(120, m_tickIn));

			wnd.draw(m_overlay);
			wnd.draw(m_infoTxt);
		}
		else
		{
			if (m_tickIn > -105)
				m_tickIn -= 10;
			m_overlay.setPosition(70.0f, m_tickIn-5);
			m_infoTxt.setPosition(sf::Vector2f(120, m_tickIn));

			wnd.draw(m_overlay);
			wnd.draw(m_infoTxt);
		}
		wnd.draw(m_map);
		wnd.draw(m_point);
	}
	bool Hud::Update(const sf::Time& tm)
	{
		auto p = m_hero->Position();
		auto dx = 3*static_cast<float>(p.x / 32);
		auto dy = 3*static_cast<float>((8000 - p.y) / 32);
		m_map.setTextureRect(sf::IntRect(256+dx - 128, 512 -dy-128, 256, 256));

		if (m_countDownTime <= 0)
			m_countDownTime = 0;
		else
			m_countDownTime -= (tm-m_prevTime).asMilliseconds();

		m_prevTime = tm;
		return true;
	}
	void Hud::TextDisplay(const std::string& txt, int time)
	{
		m_infoTxt.setString(txt);
		m_countDownTime = time;
	}
	void Hud::ShowInfoText(bool force)
	{
		m_forceShowText = force;
	}
}