#include "stdafx.h"
#include "InputManager.h"
#include "HookedRenderWindow.h"
#include <SFML/Graphics.hpp>

namespace
{
	enum class JoyDirs
	{
		None = 0,
		Up = 1,
		Down = 2,
		Right = 4,
		Left = 8,
		Button_A = 16,
		Button_X = 32,
		Button_B = 64,
		RightButton = 128,
		LeftButton = 256,
		Start = 512,
		Button_Y = 1024,
	};

	template <typename T>
	bool IsSet(T total, T check)
	{
		return static_cast<T>(static_cast<int>(total) & static_cast<int>(check)) == check;
	}

	template <typename T>
	T Set(T total, T set)
	{
		return static_cast<T>(static_cast<int>(total) | static_cast<int>(set));
	}

	JoyDirs GetJoyAction(int joystick)
	{
		if (joystick == -1)
			return JoyDirs::None;

		JoyDirs tot = JoyDirs::None;
		auto axx = sf::Joystick::getAxisPosition(joystick, sf::Joystick::PovX)
			+ sf::Joystick::getAxisPosition(joystick, sf::Joystick::X)
			+ sf::Joystick::getAxisPosition(joystick, sf::Joystick::U);

		//auto s = std::to_wstring(axx)+std::wstring(L"\r\n");
		//::OutputDebugString(s.c_str());

		if (axx < -20)
			tot = Set(tot, JoyDirs::Left);
		else if (axx > 20)
			tot = Set(tot, JoyDirs::Right);

		auto axy = sf::Joystick::getAxisPosition(joystick, sf::Joystick::PovY)
			- sf::Joystick::getAxisPosition(joystick, sf::Joystick::Y)
			- sf::Joystick::getAxisPosition(joystick, sf::Joystick::V);
		if (axy < -20)
			tot = Set(tot, JoyDirs::Down);
		else if (axy > 20)
			tot = Set(tot, JoyDirs::Up);

		if (sf::Joystick::isButtonPressed(joystick, 0)) //A
		{
			tot = Set(tot, JoyDirs::Button_A);
		}

		if (sf::Joystick::isButtonPressed(joystick, 2)) //X
			tot = Set(tot, JoyDirs::Button_X);

		if (sf::Joystick::isButtonPressed(joystick, 3)) //X
			tot = Set(tot, JoyDirs::Button_Y);

		if (sf::Joystick::isButtonPressed(joystick, 1)) //B
			tot = Set(tot, JoyDirs::Button_B);

		if (sf::Joystick::isButtonPressed(joystick, 5)) //RB
			tot = Set(tot, JoyDirs::RightButton);

		if (sf::Joystick::isButtonPressed(joystick, 4)) //LB
			tot = Set(tot, JoyDirs::LeftButton);

		if (sf::Joystick::isButtonPressed(joystick, 7)) //START
			tot = Set(tot, JoyDirs::Start);
		return tot;
	}

	//4=LB, 5 = rb, 6=BACK, 7 = START, 8 = left thumbstick button, 9 = right thumbstick button
	//0=a, 1=B, 2=x, 3=Y, PovX on left-right pad, PovY = up down pad 100=up=right, leftStick =X/Y, rightstick  = U/V
	//auto axx = sf::Joystick::getAxisPosition(firstConnectedJoy, sf::Joystick::R);
	//auto s = std::to_wstring(axx)+std::wstring(L"\r\n");
	//::OutputDebugString(s.c_str());
	//deducted = sf::Joystick::isButtonPressed(firstConnectedJoy, 10) ? 1.0 : 0.0;
}

namespace LD46
{
	InputManager::InputManager(HookedRenderWindow& window)
		:m_window(window)
		,m_detectedJoystick(-1)
		, m_action(InputAction::None)
	{
	}

	InputManager::~InputManager()
	{
	}

	void InputManager::Update()
	{
		m_action = InputAction::None;
		sf::Event event;
		auto joyDirs = GetJoyAction(m_detectedJoystick);
		while (m_window.pollEvent(event))
		{
			if (event.type == sf::Event::Closed)
				m_window.close();

			if (event.type == sf::Event::KeyPressed)
			{
				if (event.key.code == sf::Keyboard::Escape)
				{
					m_window.close();
					m_action = InputAction::ExitIngame;
				}
			}
			if (event.type == sf::Event::KeyReleased || event.type == sf::Event::JoystickButtonReleased)
			{
				if (event.key.code == sf::Keyboard::E || event.joystickButton.button == 5)
				{
					m_action = InputAction::RightButton;
				}
				if (event.key.code == sf::Keyboard::L)
					m_action = InputAction::Skipper;

				//if (event.key.code == sf::Keyboard::O)
				//	m_action = InputAction::ExitIngame;

				if (event.key.code == sf::Keyboard::Z || event.joystickButton.button == 4)
				{
					m_action = InputAction::LeftButton;
				}
			}
		}

		if (sf::Keyboard::isKeyPressed(sf::Keyboard::A) || IsSet(joyDirs, JoyDirs::Left) || sf::Keyboard::isKeyPressed(sf::Keyboard::Left))
		{
			m_action = Set(m_action, InputAction::Left);
		}

		if (sf::Keyboard::isKeyPressed(sf::Keyboard::D) || IsSet(joyDirs, JoyDirs::Right) || sf::Keyboard::isKeyPressed(sf::Keyboard::Right))
		{
			m_action = Set(m_action, InputAction::Right);
		}

		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Space) || IsSet(joyDirs, JoyDirs::Button_A))
		{
			m_action = Set(m_action, InputAction::MainButton_A);
		}

		if (sf::Keyboard::isKeyPressed(sf::Keyboard::F) || IsSet(joyDirs, JoyDirs::Button_X))
		{
			m_action = Set(m_action, InputAction::Button_X);
		}
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::G) || IsSet(joyDirs, JoyDirs::Button_Y))
		{
			m_action = Set(m_action, InputAction::Button_Y);
		}

		if (sf::Keyboard::isKeyPressed(sf::Keyboard::W) || IsSet(joyDirs, JoyDirs::Up) || sf::Keyboard::isKeyPressed(sf::Keyboard::Up))
		{
			m_action = Set(m_action, InputAction::Up);
		}
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::S) || IsSet(joyDirs, JoyDirs::Down) || sf::Keyboard::isKeyPressed(sf::Keyboard::Down))
		{
			m_action = Set(m_action, InputAction::Down);
		}
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::C) || IsSet(joyDirs, JoyDirs::Button_B))
		{
			m_action = Set(m_action, InputAction::BackButton_B);
		}

	}

	bool InputManager::IsAction(InputAction action) const
	{
		return IsSet(m_action, action);
	}

	int InputManager::DetectJoystick()
	{
		for (int i = 0; i != 8; ++i)
		{
			if (sf::Joystick::isConnected(i))
			{
				m_detectedJoystick = i;
				return m_detectedJoystick;
			}
		}

		m_detectedJoystick = -1;
		return -1;
	}
}