#include "stdafx.h"
#include <SFML/Graphics.hpp>

#include "Level.h"
#include "TextureManager.h"
#include "FontTextMgr.h"
#include "HookedRenderWindow.h"
#include "Resource.h"
#include "Hero.h"
#include "Enemy.h"
#include "Utils.h"
#include "SoundManager.h"
#include "ParticleMgr.h"

namespace LD46
{
	Level::Level(TextureManager& txMgr, SoundManager& soundMgr, FontTextMgr& fontTextMgr, ParticleMgr& partMgr)
		:m_txMgr(txMgr)
		,m_partMgr(partMgr)
		,m_ticky(0.0f)
		,m_ticker(0)
		,m_rotation(0.0f)
		,m_sndMgr(soundMgr)
		,m_fontTextMgr(fontTextMgr)
		,m_levelComplete(false)
		,m_prevHeight(0.0f)
		,m_centerHeight(0.0f)
		, m_squareScale(1.0f)
	{
		m_posFraction.x = 0;
		m_posFraction.y = 3 * 16;
		//std::array<unsigned int, MapTotalSize> m_heights;
		//std::array<unsigned int, MapTotalSize> m_map;
		//std::array<std::shared_ptr<Enemy>, MapTotalSize> m_enemies;
		m_heights.resize(MapTotalSize);
		m_map.resize(MapTotalSize);
		m_enemies.resize(MapTotalSize);

		//m_enemies.resize(MapTotalSize);
		for (int i = 0; i != MapTotalSize; ++i)
		{			
			m_enemies[i] = nullptr;
		}

		//for (int i = 0; i != Enemy::EnemyType::COUNT; ++i)
		//	m_lastSpawnTime[i];

		m_noiseTexture = m_txMgr.GetTexture(IDB_NOISE);
		m_noiseTexture.setSmooth(true);
		m_noiseImage =  m_noiseTexture.copyToImage();
		m_spr.setTexture(m_txMgr.GetTexture(IDB_PILLAR));

		for (auto& m : m_map)
			m = 0;

		LoadVoxelObject(IDB_ROUNDPILLAR, 1);
		LoadVoxelObject(IDB_PYRAMID, 2);
		LoadVoxelObject(IDB_SPIKE, 3);
		LoadVoxelObject(IDB_CUBE, 4);
		LoadVoxelObject(IDB_DOME, 5);
		LoadVoxelObject(IDB_FORK, 6);
		LoadVoxelObject(IDB_ROCK, 7);
		LoadVoxelObject(IDB_SKULL, 8);
		LoadVoxelObject(IDB_HAND, 9);
		LoadVoxelObject(IDB_GARGOYLYSPAWN, 10);
		LoadVoxelObject(IDB_BATSPAWN, 11);
		LoadVoxelObject(IDB_BLOBSPAWN, 12);
		LoadVoxelObject(IDB_BOSSSPAWN, 13);
		LoadVoxelObject(IDB_DEMONSPAWN, 14);
		LoadVoxelObject(IDB_SPIDERSPAWN, 15);
		LoadMap(IDB_WORLDMAP1);

		for (int y = 0; y != 256; ++y)
		{
			for (int x = 0; x != 256; ++x)
			{
				auto h =  20+20 * cos(3.14 * static_cast<float>(sin(static_cast<float>(x) * 0.03f * 2) - cos(static_cast<float>(y) * 0.04 * 2)) * 0.5f);
				h += 20 * cos(3.14 * static_cast<float>(x+y) * 0.02);
				if (h < 0)
					h = 0;
				//h = 0;
				//if (x > 128 && x < 160 && y > 32 && y < 64)
				//{
				//	SetMap(x / 32, y / 32, 1);					
				//}
				SetHeight(x, y, h);
				SetParticle(x, y, 0);
			}
		}
	}
	Level::~Level()
	{
	}
	void Level::LoadMap(int resourceId)
	{
		char tileConverter[64] = { 0,1,2,3,4,5,6,7,8,0,9,10,11,12,13,14,15,
								  16 + 0,16 + 1,16 + 2,16 + 3,16 + 4,16 + 5,16 + 6,16 + 7,16 + 8,16 + 9,16 + 10,16 + 11,16 + 12,16 + 13,16 + 14, 16 + 15 };
		auto csvTiledMap = m_fontTextMgr.LoadTextInResource(resourceId);
		auto startIndex = 0;
		auto lineIter = csvTiledMap.find('\n');
		int mapY = 0;
		while (lineIter != -1)
		{
			auto line = csvTiledMap.substr(startIndex, lineIter - startIndex);
			auto numStart = 0;
			startIndex = ++lineIter;
			auto numIter = line.find(',');
			int mapX = 0;
			while (numIter != -1)
			{
				auto num = line.substr(numStart, numIter - numStart);
				auto nr = std::stoi(num);
				SetMap(mapX, mapY, tileConverter[nr - 16]);
				numStart = ++numIter;
				numIter = line.find(',', numStart);
				++mapX;
			}
			lineIter = csvTiledMap.find('\n', startIndex);
		    ++mapY;
		}
	}

	void Level::LoadVoxelObject(int resourceId, int textureIndex)
	{
		auto& t = m_txMgr.GetTexture(resourceId);
		t.setSmooth(false);
		auto& img = t.copyToImage();
		auto& tex = m_texture[textureIndex];
		for (int x = 0; x != 32; ++x)
		{
			for (int y = 0; y != 32; ++y)
			{
				auto prevPx = false;
				int prevCnt = 0;
				int layCnt = 0;
				//int leftOver = 32;
				auto& line = tex[x + y * 32];
				for (int layer = 0; layer != 32; ++layer)
				{
					auto& c = img.getPixel(x, y + layer * 32);
					auto px = (c.r != 0);// ? 255 : 0;
					if (px != prevPx)
					{
						//line[layCnt++] = prevCnt;
						line.push_back(prevCnt);
						//leftOver -= prevCnt;
						prevCnt = 0;
						prevPx = px;
					}
					else
					{
						++prevCnt;
					}
				}
				//if (layCnt == 0)
					//line[layCnt] = prevCnt;
				line.push_back(prevCnt);
			}
		}
	}

	void Level::HeroShoot()
	{
		if (m_hero->GetWorldState() != WorldState::Chaos)
			return;

		sf::Vector3f ppos(m_posFraction.x, 5, m_posFraction.y);
		auto pId = m_partMgr.CreateParticle(ppos, ParticleMgr::ParticleType::HeroShot,2000);
		SetParticle(0.5 * m_posFraction.x, 0.5 * m_posFraction.y, pId);
		m_heroShotDir[pId] = sf::Vector3f(-5.0*sin(m_rotation), 0.1,-5.0* cos(m_rotation));
	}

	std::vector<int> Level::GetHeroShots() const
	{
		std::vector<int> v;
		for (const auto& hs : m_heroShotDir)
			v.push_back(hs.first);

		return v;
	}

	void Level::RemoveHeroShots(const std::vector<int>& shotsIds)
	{
		for (auto sId : shotsIds)
		{
			auto iter = m_heroShotDir.find(sId);
			if (iter != std::end(m_heroShotDir))
				m_heroShotDir.erase(iter);

			m_partMgr.RemoveParticle(sId);
		}
	}

	bool Level::Update(const sf::Time& tm)
	{
		m_currentTime = tm;
		m_ticky += 0.1f;
		++m_ticker;
		m_posFraction.x = 134.0f;
		//m_posFraction.x += 1.0f;
		m_posFraction.y = 60.0f;
		//m_posFraction.y -= 0.5f;
		m_posFraction = m_hero->Position();
		m_pos.x = m_posFraction.x;
		m_pos.y = m_posFraction.y;
		m_rotation = m_hero->Rotation();
		std::vector<unsigned int> particlesToRemove;
		for (auto& p : m_heroShotDir)
		{
			auto pId = p.first;
			if (m_partMgr.Particle(pId) == nullptr)
			{
				particlesToRemove.push_back(pId);
			}
			else
			{
				auto ppos = m_partMgr.ParticlePos(pId);
				auto pId = Particle(ppos.x, ppos.z);;
				if (pId == p.first) //Yes this is the right particle
				{
					const auto& dir = p.second;
					SetParticle(0.5 * ppos.x, 0.5 * ppos.z, 0);
					ppos += dir;
					m_partMgr.SetParticlePos(pId, ppos);
					SetParticle(0.5 * ppos.x, 0.5 * ppos.z, pId);
				}
			}
		}
		for (auto r : particlesToRemove)
		{
			m_heroShotDir.erase(r);
		}

//		m_rotation += 0.02f;
		//auto h = Height(m_pos.x, m_pos.y);
		//auto ss = 32.0f + cos(m_ticky)*32.0f;
		//SetHeight(0.5* m_posFraction.x, 0.5 * m_posFraction.y, ss);
		//auto pId = Particle(m_posFraction.x, m_posFraction.y);
		//if (pId == 0)
		//{
		//	pId = m_partMgr.CreateParticle(sf::Vector3f(m_posFraction.x, 16, m_posFraction.y), ParticleMgr::ParticleType::Slash);
		//	SetParticle(0.5 * m_posFraction.x, 0.5 * m_posFraction.y, pId);
		//}
		return false;
	}

	void Level::Draw(sf::RenderWindow& wnd, sf::Shader& shdr, sf::Shader& skyShdr)
	{
//		m_noisePos.x = 0.3;
	//	m_noisePos.y = 0.3;
		m_noisePos.x += 0.004 * cos(m_ticky * 0.003);
		m_noisePos.y += 0.004 * sin(m_ticky * 0.004);
		m_noisePos2.x += 0.002 * cos(m_ticky * 0.002);
		m_noisePos2.y += 0.002 * sin(m_ticky * 0.001);

		m_skyNoisePos.x += 0.004 * cos(m_ticky * 0.003);
		m_skyNoisePos.y += 0.004 * sin(m_ticky * 0.004);
		m_skyNoisePos2.x += 0.002 * cos(m_ticky * 0.002);
		m_skyNoisePos2.y += 0.002 * sin(m_ticky * 0.001);

		auto py = m_pos.y;
		int x1 =0, y1 = 0;
		int height = Height(x1, y1);
		sf::RectangleShape rec(sf::Vector2f(4.0f, 4.0f));
		rec.setOrigin(2.0f, 2.0f);
		rec.setFillColor(sf::Color(255, 0, 255, 255));
		//rec.setPosition()
		sf::Vector2f scale;		
		sf::VertexArray vertexArray(sf::Quads, 100000);
		sf::Color col1(128, 0, 0, 255);
		sf::Color col2(0, 0, 0, 255);
		sf::Color colZero(0, 0, 0, 255);
		vertexArray.append(sf::Vertex(sf::Vector2f(0, 0)));
		vertexArray.append(sf::Vertex(sf::Vector2f(640, 0)));
		vertexArray.append(sf::Vertex(sf::Vector2f(640, 400)));
		vertexArray.append(sf::Vertex(sf::Vector2f(0, 400)));
		skyShdr.setUniform("noisePos", m_skyNoisePos);
		skyShdr.setUniform("noisePos2", m_skyNoisePos2);
		auto rotor = m_rotation;
		while (rotor > 3.14 * 2)
			rotor -= 3.14*2;
		auto rr = 2.0*rotor/3.14 - 0.5;
		sf::Vector2f sunPos(rr, 0.8);
		skyShdr.setUniform("sunPos", sunPos);
		sf::RenderStates skyStates;
		skyStates.shader = &skyShdr;
		skyStates.texture = &m_noiseTexture;
		wnd.draw(vertexArray, skyStates);

		vertexArray.clear();

		auto xpos = m_pos.x;
		auto yStepX = sin(m_rotation);
		auto yStepY = cos(m_rotation);
		auto xStepX = sin(m_rotation + 3.14f * 0.5f);
		auto xStepY = cos(m_rotation + 3.14f * 0.5f);

		auto yPosX = m_posFraction.x - yStepX * 250;
		auto yPosY = m_posFraction.y - yStepY * 250;
		auto iterX = 0;
		auto iterY = 0.0f;
		std::set<int> enemiesAlreadyRendered;
		std::set<int> particlesAlreadyRendered;
		shdr.setUniform("noisePos", m_noisePos);
		shdr.setUniform("noisePos2", m_noisePos2);
		shdr.setUniform("position", m_hero->Position());
		shdr.setUniform("texRotation", m_rotation);
		sf::RenderStates states;
		m_prevHeight = m_centerHeight;
		m_squareScale = m_hero->GetWorldState() == WorldState::Shadow ? 0.5f : 1.0f;		
		states.shader = &shdr;
		states.texture = &m_noiseTexture;
		auto closestPos = 10000000.0f;
		
		for (int z = 0; z != 120; ++z)
		{
			
			sf::Vector2f pos;
			pos.x = 0;
			auto yBase = 100 + z * (1.0f + 0.02f * static_cast<float>(z));//pow(static_cast<float>(z) * 0.2f, 2.0);
			pos.y = yBase;
			scale.x = 4.0f * (static_cast<float>(z) * 0.04f* m_squareScale + 0.01f);
			scale.y = scale.x*0.99;
			rec.setScale(scale);
			auto stepX = static_cast<float>(z) * 0.1f + 1.0f;
			//stepX = sqrtf(stepX)*2;
			auto heightScale = (z+1.0)*0.02f* m_squareScale;// (3.0 + (log(static_cast<float>(z + 1) * 0.1f))) / 3.0f;
			//heightScale = heightScale * heightScale * 0.8;
			//heightScale = sqrtf(heightScale)*2;
			if (heightScale < 0)
				pos.x = 320.0f + 1;
			iterX = 0;

			auto xPosX = yPosX;
			auto xPosY = yPosY;
			for (float x = 0; x < 320; x += stepX)
			{
				pos.x = 320.0f + x;
				DrawHeight(iterY, z, 0, 0, pos, x, xPosX, iterX, xStepX, xPosY, xStepY, yBase, scale, vertexArray, heightScale, wnd, enemiesAlreadyRendered, particlesAlreadyRendered, closestPos);
				pos.x = 320.0f - x;
				DrawHeight(iterY, z, 0, 0, pos, x, xPosX, -iterX, xStepX, xPosY, xStepY, yBase, scale, vertexArray, heightScale, wnd, enemiesAlreadyRendered, particlesAlreadyRendered, closestPos);
				++iterX;
			}
			auto stepAdjust = (static_cast<float>(100-z) * 0.05);
			if (stepAdjust < 1.0)
				stepAdjust = 1.0;
			iterY += stepAdjust;
			yPosX += yStepX*stepAdjust;
			yPosY += yStepY * stepAdjust;

			wnd.draw(vertexArray, states);
			vertexArray.clear();
		}		

/*		shdr.setUniform("noisePos", m_noisePos);
		shdr.setUniform("noisePos2", m_noisePos2);
		shdr.setUniform("position", m_hero->Position());
		shdr.setUniform("texRotation", m_rotation);
		sf::RenderStates states;
		states.shader = &shdr;
		states.texture = &m_noiseTexture;*/
		//wnd.draw(vertexArray, states);
		m_hero->Draw(wnd);
		//if (m_hero->IsAction(Action::Standing))
		{
			sf::VertexArray vertexArray2(sf::Triangles, 3);
			sf::Color ccr(255, 0, 0, 255);//339,296 -> 350, 326 -> 328,326
			sf::Color ccb(0, 255, 0, 255);//339,296 -> 350, 326 -> 328,326
			sf::Color ccg(0, 0, 255, 255);//339,296 -> 350, 326 -> 328,326
			sf::Color ccbl(0, 0, 0, 255);//339,296 -> 350, 326 -> 328,326
			auto chaosScale = m_hero->Power(WorldState::Chaos); //sin(m_ticky) * 0.5 + 0.5;
			auto darkScale = m_hero->Power(WorldState::Darkness); //cos(m_ticky) * 0.5 + 0.5;
			auto shadowScale = m_hero->Power(WorldState::Shadow); //cos(m_ticky*3.3) * 0.5 + 0.5;

			vertexArray2.append(sf::Vertex(sf::Vector2f(339, 311 - 15*chaosScale), ccr));
			vertexArray2.append(sf::Vertex(sf::Vector2f(339, 311), ccr)); //center ,red,
			vertexArray2.append(sf::Vertex(sf::Vector2f(339 - 11*chaosScale, 311 + 15*chaosScale), ccr));
			vertexArray2.append(sf::Vertex(sf::Vector2f(339, 311 - 15 * darkScale), ccb));
			vertexArray2.append(sf::Vertex(sf::Vector2f(339+11*darkScale, 311 + 15 * darkScale), ccb));
			vertexArray2.append(sf::Vertex(sf::Vector2f(339, 311), ccb));
			vertexArray2.append(sf::Vertex(sf::Vector2f(339 + 11 * shadowScale, 311 + 15 * shadowScale), ccg));
			vertexArray2.append(sf::Vertex(sf::Vector2f(339, 311), ccg));
			vertexArray2.append(sf::Vertex(sf::Vector2f(339 - 11 * shadowScale, 311 + 15 * shadowScale), ccg));
			//vertexArray2.append(sf::Vertex(sf::Vector2f(339, 304), ccbl));
			//vertexArray2.append(sf::Vertex(sf::Vector2f(344, 320), ccbl));
			//vertexArray2.append(sf::Vertex(sf::Vector2f(334, 320), ccbl));
			wnd.draw(vertexArray2);
		}
		//::OutputDebugStringA((std::to_string(sx) + ":"+ std::to_string(sy)+"\r\n").c_str());
		//m_sndMgr.PlaySound(IDS_POTIONPICKUP);
		//m_hero->SetObjectCount(ObjectType::Key, m_hero->ObjectCount(ObjectType::Key) - 1);
	}

	void Level::DrawHeight(float iterY, int z, int dxx, int dxy, sf::Vector2f& pos, float x, float xPosX, int iterX, float xStepX, float xPosY, float xStepY, float yBase, sf::Vector2f& scale, sf::VertexArray& vertexArray, double heightScale, sf::RenderWindow& wnd, std::set<int>& enemiesAlreadyRendered, std::set<int>& particlesAlreadyRendered, float& closestPos)
	{
		int dx = xPosX + iterX * xStepX;
		int dy = xPosY + iterX * xStepY;
		//auto colb = m_noiseImage.getPixel((dx) & 0xff, (dy) & 0xff).r;
		auto alphas = 255;
		auto h = Height(dx, dy);
		auto worldState = m_hero->GetWorldState();
		if (worldState != WorldState::Chaos)
			h = 0;
		else
		{

			h += sin(sin(iterY*0.03) + sin(iterX*0.07) + m_ticky*2)*40;
		}

		if (z == 60 && x == 0)
		{
			m_centerHeight = h;
		}
		auto heightAdjust = h * heightScale;
		//auto zcol = 255 - sqrtf(h * h * 3 + (200 - z * 2) * (200 - z * 2) + (iterX * 2) * (iterX * 2)) * 1.5;
		auto m = Map(dx / 32, dy / 32);
		auto enemyPtr = GetEnemy(dx / 32, dy / 32);
		if (enemyPtr != nullptr)
		{
			//if (qq < closestPos)
			//	closestPos = qq;

			
			if (enemiesAlreadyRendered.find(enemyPtr->Id()) == std::end(enemiesAlreadyRendered))
			{
				auto& epos = enemyPtr->Position();
				auto qx = dx - epos.x;
				auto qy = dy - epos.z;
				auto qq = qx * qx + qy * qy;
				auto thrshld = 0.2*(100.0f - static_cast<float>(z))+2.0f;
				if (qq < thrshld)
				{
					//This should really also check the exact line that the sprite should be rendered at....
					enemiesAlreadyRendered.insert(enemyPtr->Id());
					auto py = yBase - epos.y * (scale.y * 0.7f / m_squareScale) - heightAdjust;
					enemyPtr->SetSpritePosition(sf::Vector2f(pos.x, py));
					enemyPtr->SetScale(scale / m_squareScale);
					enemyPtr->Draw(wnd);
				}
			}
		}
		auto particleId = Particle(dx, dy);
		if (particleId != 0)
		{
			if (particlesAlreadyRendered.find(particleId) == std::end(particlesAlreadyRendered))
			{
				auto* particle = m_partMgr.Particle(particleId);
				if (particle != nullptr)
				{
					auto& epos = m_partMgr.ParticlePos(particleId);
					auto qx = dx - epos.x;
					auto qy = dy - epos.z;
					auto qq = qx * qx + qy * qy;
					if (qq < 5.0)
					{
						//This should really also check the exact line that the sprite should be rendered at....
						particlesAlreadyRendered.insert(particleId);
						auto py = yBase - epos.y * scale.y * 0.7f - heightAdjust;
						auto sc = sf::Vector2f(scale.x * 0.3, scale.y * 0.3);
						particle->SetPos(sf::Vector2f(pos.x, py));
						particle->SetScale(sc);
						particle->Draw(wnd);
					}
				}
			}
		}

		auto zcol = (100 - z)*3;
		if (m != 0)
		{
			if ((m & 16) == 16)
				alphas = worldState == WorldState::Shadow ? 16 : 255;

			m = m & 15;

			Enemy::EnemyType spawnType = Enemy::EnemyType::COUNT;
			switch (m)
			{
			case 10:
				spawnType = Enemy::EnemyType::Gargoyle;
				break;
			case 11:
				spawnType = Enemy::EnemyType::Bat;
				break;
			case 12:
				spawnType = Enemy::EnemyType::Blob;
				break;
			case 13:
			{
				//This like everything else, is severely broken..
				int ix = dx / 32;
				int iy = dy / 32;
				if (ix == 12 && iy == 195)
					spawnType = Enemy::EnemyType::Boss3;
				else if (ix == 39 && iy == 210)
					spawnType = Enemy::EnemyType::Boss2;
				else if (ix == 71 && iy == 234)
					spawnType = Enemy::EnemyType::Boss1;
				else
					spawnType = Enemy::EnemyType::Bug;
			}
				break;
			case 14:
				spawnType = Enemy::EnemyType::Demon;
				break;
			case 15:
				spawnType = Enemy::EnemyType::Bug;
				break;
			default:
				spawnType = Enemy::EnemyType::COUNT;
				break;
			}

			if (spawnType != Enemy::EnemyType::COUNT && (m_currentTime - m_lastSpawnTime[spawnType]).asMilliseconds() > 500)
			{
				m_enemeisToSpawn[spawnType] = sf::Vector3f(dx, 8, dy);
				m_lastSpawnTime[spawnType] = m_currentTime;
			}
			//sf::Color col(static_cast<int>(dx) * 25 | static_cast<int>(dy) * 10, 0, 0, 255);
			sf::Color col(dy, dx, 0,  alphas);
			//sf::Color col(0, 0, 0, alphas);
			auto sy = scale.y * 0.7f/ m_squareScale;
			pos.y = yBase-32 * sy - heightAdjust;
			auto& tex = m_texture[m];
			auto& l = tex[(dx & 31) + (dy & 31) * 32];
			int sd = 0;
			int d = l[sd++];
			pos.y += sy * d;
			//for (int d = 0; d < 32; ++d)
			while (d < 32)
			{
				if (sd >= l.size())
					break;

				auto len = l[sd++];
				//if (l[d] != 0)
				{
					//col.b =  d;
					vertexArray.append(sf::Vertex(pos, col));
					//col.g += scale.x;
					vertexArray.append(sf::Vertex(sf::Vector2f(pos.x + scale.x, pos.y), col));
					//col.g -= scale.x;
					//col.r += sy * len;
					vertexArray.append(sf::Vertex(sf::Vector2f(pos.x + scale.x, pos.y + sy*len), col));
					//col.g += scale.x*0.1;
					vertexArray.append(sf::Vertex(sf::Vector2f(pos.x, pos.y + sy*len), col));
				}
				if (sd >= l.size())
					break;

				auto emptySpace = l[sd++] +len;
				pos.y += sy*emptySpace;
				d += emptySpace;
			}
		}
		//else
		{
			//if (zcol < 255)
			{
				//sf::Color col(0, 0, static_cast<int>(dx) * 25 | static_cast<int>(dy) * 10, 255);
				sf::Color col(dy, dx, 255/*heightAdjust*/, 255);
				//sf::Color col(0, 0, 0, 255);
				pos.y = yBase - heightAdjust;
				vertexArray.append(sf::Vertex(pos, col));
				vertexArray.append(sf::Vertex(sf::Vector2f(pos.x + scale.x, pos.y), col));
				if (m_hero->GetWorldState() == WorldState::Chaos)
					pos.y = yBase;

				//col.r += scale.y;
				vertexArray.append(sf::Vertex(sf::Vector2f(pos.x + scale.x, pos.y + scale.y), col));
				vertexArray.append(sf::Vertex(sf::Vector2f(pos.x, pos.y + scale.y), col));
			}
		}
	}



	void Level::SetHero(std::shared_ptr<Hero> hero)
	{
		m_hero = hero;
		//Setup initial values.
		m_hero->SetSpritePoition(sf::Vector2f(292, 280));
	}

	void Level::SetPos(const sf::Vector2i& pos)
	{
		m_pos = pos;
	}

	unsigned int Level::Particle(const int x, const int y) const
	{
		auto xx = x / 2;
		auto yy = y / 2;

		return (m_heights[(xx & 0xff) + (yy & 0xff) * 256] >> 16) & 0xffff;
	}

	void Level::SetParticle(const int x, const int y, unsigned int particleId)
	{
		auto& h = m_heights[(x & 0xff) + (y & 0xff) * 256];
		h = (particleId << 16 & 0xffff0000) | (h & 0xffff);
	}

	int Level::Height(const int x, const int y) const
	{
		auto xx = x / 2;
		auto yy = y / 2;

		return m_heights[(xx&0xff) + (yy&0xff) * 256] & 0xffff;
	}

	void Level::SetHeight(const int x, const int y, int height)
	{
		auto& h = m_heights[(x & 0xff) + (y & 0xff) * 256];
		h = (height & 0xffff) | (h & 0xffff0000);
	}


	int Level::Map(const int x, const int y) const
	{
		auto xx = x;
		auto yy = y;
	/*	while (xx < 0)
			xx += 256;
		while (yy < 0)
			yy += 256;*/

		return m_map[(xx & 0xff) + (yy & 0xff) * 256];
	}

	const std::map<int, sf::Vector3f>& Level::SpawnRequest() const
	{
		return m_enemeisToSpawn;
	}

	void Level::ClearSpawnRequest()
	{
		m_enemeisToSpawn.clear();
	}


	void Level::SetMap(const int x, const int y, int height)
	{
		if (x < 0 || y < 0 || y > 255 || x > 255)
			return;

		m_map[x + y * 256] = height;
	}

	std::shared_ptr<Enemy> Level::GetEnemy(const int x, const int y) const
	{
		auto xx = x;
		auto yy = y;
/*		while (xx < 0)
			xx += 256;
		while (yy < 0)
			yy += 256;*/

		return m_enemies[(xx & 0xff) + (yy & 0xff) * 256];
	}


	void Level::SetEnemy(const int x, const int y, std::shared_ptr<Enemy> enemy)
	{
/*		if (x < 0 || y < 0 || y > 255 || x > 255)
			return;
			*/
		m_enemies[(x&0xff) + (y & 0xff) * 256] = enemy;
	}


	//bool Level::SetEnemy(const sf::Vector3f& pos, LD46::Enemy* enemy)
	//{
	//	if (pos.x < 0 || pos.y < 0 || pos.z < 0)
	//		return false;
	//	return true;
	//}

	void Level::ClearEnemy(const sf::Vector3f& pos)
	{
		SetEnemy(pos.x / 32, pos.z / 32, nullptr);
		//if (pos.x < 0 || pos.y < 0 || pos.z < 0)
		//	return;
		//m_enemies[x + y * 256] = nullptr;
	}

	sf::Vector3f Level::HeroPos() const
	{
		return sf::Vector3f(m_posFraction.x, 8.0,  m_posFraction.y);
	}

	bool Level::IsLevelComplete() const
	{
		return m_levelComplete;
	}

	bool Level::CanWalk(float x, float y)
	{
		const auto& m = Map(x / 32, y / 32);
		if (m_hero->GetWorldState() == WorldState::Shadow)
			return m > 16 || m == 0 || m == 13;
		
		return m == 0 || m == 13;
	}
}