#pragma once
#include <SFML/Graphics.hpp>
#include <array>
#include <set>

namespace LD46
{
	class HookedRenderWindow;
	class TextureManager;
	class Hero;
	class Enemy;
	class SoundManager;
	class FontTextMgr;
	class ParticleMgr;

	class Level
	{
	public:

		Level(TextureManager& txMgr, SoundManager& soundMgr, FontTextMgr& fontTextMgr, ParticleMgr& partMgr);
		~Level();

		bool Update(const sf::Time& tm);
		void Draw(sf::RenderWindow& wnd, sf::Shader& shdr, sf::Shader& skyShdr);
		void DrawHeight(float iterY, int z, int dx, int dy, sf::Vector2f& pos, float x, float xPosX, int iterX, float xStepX, float xPosY, float xStepY, float yBase, sf::Vector2f& scale, sf::VertexArray& vertexArray, double heightScale, sf::RenderWindow& wnd, std::set<int>& enemiesAlreadyRendered, std::set<int>& particlesAlreadyRendered, float& closestPos);
		void SetHero(std::shared_ptr<Hero> spr);
		//bool SetEnemy(const sf::Vector3f& pos, Enemy* enemy);
		void SetEnemy(const int x, const int y, std::shared_ptr<Enemy> enemy);
		std::shared_ptr<Enemy> GetEnemy(const int x, const int y) const;
		bool CanWalk(float x, float y);
		void ClearEnemy(const sf::Vector3f& pos);
		sf::Vector3f HeroPos() const;
		bool IsLevelComplete() const;
		int Height(const int x, const int y) const;
		unsigned int Particle(const int x, const int y) const;
		int Map(const int x, const int y) const;
		const std::map<int, sf::Vector3f>& SpawnRequest() const;
		void ClearSpawnRequest();
		void HeroShoot();
		std::vector<int> GetHeroShots() const;
		void RemoveHeroShots(const std::vector<int>& shotsIds);
		void SetParticle(const int x, const int y, unsigned int particle);
		void SetMap(const int x, const int y, int height);
	private:
		void LoadMap(int resourceId);
		void LoadVoxelObject(int resourceId, int textureIndex);
		void SetPos(const sf::Vector2i& pos);
		void SetHeight(const int x, const int y, int height);
		TextureManager& m_txMgr;
		ParticleMgr& m_partMgr;
		sf::Sprite m_spr;
		const static int MapTotalSize = 65536;
		//std::array<unsigned int, MapTotalSize> m_heights;
		//std::array<unsigned int, MapTotalSize> m_map;
		//std::array<std::shared_ptr<Enemy>, MapTotalSize> m_enemies;
		std::vector<unsigned int> m_heights;
		std::vector<unsigned int> m_map;
		std::vector<std::shared_ptr<Enemy>> m_enemies;

		//std::vector<Enemy*> m_enemies;
		std::shared_ptr<Hero> m_hero;
		sf::Vector2i m_lastHeroPos;
		float m_ticky;
		int m_ticker;
		sf::Vector2i m_pos;
		sf::Vector2f m_posFraction;
		float m_rotation;
		SoundManager& m_sndMgr;
		FontTextMgr& m_fontTextMgr;
		bool m_levelComplete;
		std::map<int, std::array<std::vector<unsigned char>, 1024>> m_texture;
		sf::Texture m_noiseTexture;
		sf::Image m_noiseImage;
		sf::Vector2f m_noisePos;
		sf::Vector2f m_noisePos2;
		sf::Vector2f m_skyNoisePos;
		sf::Vector2f m_skyNoisePos2;
		float m_prevHeight;
		float m_centerHeight;
		float m_squareScale;
		std::array<sf::Time, /*Enemy::EnemyType::COUNT*/12> m_lastSpawnTime;
		sf::Time m_currentTime;
		std::map<int, sf::Vector3f> m_enemeisToSpawn;
		std::map<int, sf::Vector3f> m_heroShotDir;
	};
}