#include "stdafx.h"
#include "MusicManager.h"
#include <string>
#include <sstream>
#include <iomanip>
#include <locale>
#include <codecvt>
#include <iostream>
#define _SILENCE_CXX17_CODECVT_HEADER_DEPRECATION_WARNING
namespace LD46
{
	void MusicManager::Play(int resourceId)
	{
		auto music = GetResource(resourceId);
		if (music != nullptr)
			music->play();
	}

	void MusicManager::Stop(int resourceId)
	{
		auto music = GetResource(resourceId);
		if (music != nullptr)
			music->stop();
	}

	MusicManager::~MusicManager()
	{
		for (auto& tempFilename : m_tempFiles)
			::DeleteFile(tempFilename.second);
	}

	sf::Music* MusicManager::GetResource(const int resourceId)
	{
		auto iter = m_songs.find(resourceId);
		if (iter != std::end(m_songs))
			return &iter->second;

		return nullptr;
	}


	sf::Music& MusicManager::LoadFromResource(int resourceId)
	{
		auto handle = ::GetModuleHandle(NULL);
		HRSRC rsrcData = FindResource(handle, MAKEINTRESOURCE(resourceId), RT_RCDATA);
		if (!rsrcData)
			throw std::runtime_error("Failed to find resource.");

		DWORD rsrcDataSize = SizeofResource(handle, rsrcData);
		if (rsrcDataSize <= 0) {
			MessageBox(NULL, L"Size of resource is 0.", L"ERROR", MB_OK);
			throw std::runtime_error("Size of resource is 0.");
		}

		HGLOBAL grsrcData = LoadResource(handle, rsrcData);
		if (!grsrcData)
			throw std::runtime_error("Failed to load resource.");

		LPVOID firstByte = LockResource(grsrcData);
		if (!firstByte)
			throw std::runtime_error("Failed to lock resource.");

		TCHAR tempPath[2048];
		auto res = ::GetTempPath(2048, tempPath);
		if (res != 0 && res <= 2048)
		{
			auto& tempFilename = m_tempFiles[resourceId];
			if (::GetTempFileName(tempPath, _T("LD44WRECK"), 0, tempFilename) != 0)
			{
				auto hFile = ::CreateFile(tempFilename, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
				DWORD bytesWritte = 0;
				::WriteFile(hFile, firstByte, rsrcDataSize, &bytesWritte, NULL);
				CloseHandle(hFile);
			}
			std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> converter;

			auto& music = m_songs[resourceId];
			auto filename = converter.to_bytes(std::wstring(tempFilename));
			if (!music.openFromFile(filename))
				throw std::runtime_error("Could not open file");

			music.setLoop(true);
			return music;
		}
		throw std::runtime_error("Resource creation failed!");
	}

}