#include "stdafx.h"
#include "Resource.h"
#include "ParticleMgr.h"
#include "TextureManager.h"
namespace LD46
{
	ParticleMgr::ParticleMgr(TextureManager& txMgr, SoundManager& soundMgr)
		:m_txMgr(txMgr)
		, m_sndMgr(soundMgr)
		, m_particleCnt(0)
	{
		//m_sprites.push_back(std::unique_ptr<AnimatedSprite>());
		//m_spritePositions.emplace_back(sf::Vector3f());
		//m_spriteLifeTimes.emplace_back(1000);
	}

	ParticleMgr::~ParticleMgr()
	{
	}
	void ParticleMgr::Update(const sf::Time& tm, Hero& hero, Level& level)
	{
		auto elapsed = (tm - prevTime).asMilliseconds();
		std::vector<int> removeSprites;
		for (auto& p : m_sprites)
		{
			if (p.second)
			{
				p.second->Update(tm);
				auto& life = m_spriteLifeTimes[p.first];
				life -= elapsed;
				if (life <= 0)
					removeSprites.push_back(p.first);
			}
		}
		for (auto p : removeSprites)
		{
			m_sprites.erase(p);
			m_spriteLifeTimes.erase(p);
			m_spritePositions.erase(p);
		}

		prevTime = tm;
	}
	unsigned int ParticleMgr::CreateParticle(const sf::Vector3f& pos, const ParticleType type, unsigned int lifeTime /*= 10000*/)
	{
		//TextureManager texMgr, const int resourceId, const int width, const int height, const float speed
		switch (type)
		{
		case ParticleType::HeroShot:
			m_sprites[m_particleCnt] = (std::make_unique<AnimatedSprite>(m_txMgr, IDB_HEROSHOT, 32, 32, 1.0, false));
			break;
		case ParticleType::Banish:
			m_sprites[m_particleCnt] = (std::make_unique<AnimatedSprite>(m_txMgr, IDB_BANISH, 64, 64, 2.0, true, 32, 32));
			break;
		case ParticleType::HitEnemy:
			m_sprites[m_particleCnt] = (std::make_unique<AnimatedSprite>(m_txMgr, IDB_ENEMYHIT, 32, 32, 2.0, true));
			break;
		case ParticleType::HitHero:
			m_sprites[m_particleCnt] = (std::make_unique<AnimatedSprite>(m_txMgr, IDB_HITHERO, 64, 64, 1.0, true, 28, 32));
			break;
		case ParticleType::Slash:
		case ParticleType::Star:
		default:
			m_sprites[m_particleCnt] = (std::make_unique<AnimatedSprite>(m_txMgr, IDB_BAT, 32, 32, 1.0, false));
			break;
		}
		auto& p = m_sprites[m_particleCnt];
		//p->SetOrigin(sf::Vector2f(16.0f, 16.0f));
		p->Play();
		m_spritePositions[m_particleCnt] = pos;
		m_spriteLifeTimes[m_particleCnt] = lifeTime;
		return m_particleCnt++;
	}

	const sf::Vector3f& ParticleMgr::ParticlePos(const unsigned int particleId) const
	{
		auto& iter = m_spritePositions.find(particleId);
		return iter->second; //no errorchecking..
		//return m_spritePositions[particleId];
	}

	void ParticleMgr::SetParticlePos(const unsigned int particleId, const sf::Vector3f& pos)
	{
		m_spritePositions[particleId] = pos;
	}

	AnimatedSprite* ParticleMgr::Particle(const unsigned int particleId)
	{
		auto iter = m_sprites.find(particleId);
		if (iter == std::end(m_sprites))
			return nullptr;

		return iter->second.get();
	}

	int ParticleMgr::ParticleLife(const unsigned int particleId) const
	{
		return m_spriteLifeTimes.find(particleId)->second;
		//return m_spriteLifeTimes[particleId];
	}
	void ParticleMgr::RemoveParticle(const unsigned int particleId)
	{
		m_sprites.erase(particleId);
		m_spriteLifeTimes.erase(particleId);
		m_spritePositions.erase(particleId);
	}
}