#pragma once
#include <SFML/Graphics.hpp>
#include <memory>
#include <map>
#include <string>
#include "AnimatedSprite.h"

namespace LD46
{
	class Level;
	class Hero;
	class TextureManager;
	class SoundManager;
	enum class CollisionType;

	class ParticleMgr
	{
	public:
		enum class ParticleType
		{
			Star,
			Slash,
			HeroShot,
			Banish,
			HitEnemy,
			HitHero
		};

		ParticleMgr(TextureManager& txMgr, SoundManager& soundMgr);
		virtual ~ParticleMgr();
		void Update(const sf::Time& tm, Hero& hero, Level& level);
		unsigned int CreateParticle(const sf::Vector3f& pos, const ParticleType type, unsigned int lifeTime = 10000);
		const sf::Vector3f& ParticlePos(const unsigned int particleId) const;
		void SetParticlePos(const unsigned int particleId, const sf::Vector3f& pos);
		AnimatedSprite* Particle(const unsigned int particleId);
		int ParticleLife(const unsigned int particleId) const;
		void RemoveParticle(const unsigned int particleId);
	private:
		TextureManager& m_txMgr;
		SoundManager& m_sndMgr;
		unsigned long m_particleCnt;
		std::map<unsigned long , std::unique_ptr<AnimatedSprite>> m_sprites;
		std::map<unsigned long, sf::Vector3f> m_spritePositions;
		std::map<unsigned long, int/*lifeTimeOfSpriteInMilliseconds*/> m_spriteLifeTimes;
		sf::Time prevTime;
	};
}