#include "stdafx.h"
#include "SoundManager.h"

namespace LD46
{
	sf::SoundBuffer SoundManager::LoadBufferFromResource(int name)
	{
		auto handle = ::GetModuleHandle(NULL);
		HRSRC rsrcData = FindResource(handle, MAKEINTRESOURCE(name), RT_RCDATA);
		if (!rsrcData)
			throw std::runtime_error("Failed to find resource.");

		DWORD rsrcDataSize = SizeofResource(handle, rsrcData);
		if (rsrcDataSize <= 0) {
			MessageBox(NULL, L"Size of resource is 0.", L"ERROR", MB_OK);
			throw std::runtime_error("Size of resource is 0.");
		}

		HGLOBAL grsrcData = LoadResource(handle, rsrcData);
		if (!grsrcData)
			throw std::runtime_error("Failed to load resource.");

		LPVOID firstByte = LockResource(grsrcData);
		if (!firstByte)
			throw std::runtime_error("Failed to lock resource.");

		sf::SoundBuffer buffer;
		if (!buffer.loadFromMemory(firstByte, rsrcDataSize))
			throw std::runtime_error("Failed to load image from memory.");

		return buffer;
	}

	void SoundManager::PlaySound(int resourceId)
	{
		if (m_buffers.find(resourceId) != std::end(m_buffers))
		{
			m_sounds[resourceId].play();
			return;
		}

		m_buffers[resourceId] = LoadBufferFromResource(resourceId);
		auto& sound = m_sounds[resourceId];
		sound.setBuffer(m_buffers[resourceId]);
		sound.play();
	}
}