#include "stdafx.h"
#include "TextureManager.h"

namespace LD46
{
	sf::Image TextureManager::LoadImageFromResource(int name)
	{
		auto handle = ::GetModuleHandle(NULL);
		HRSRC rsrcData = FindResource(handle, MAKEINTRESOURCE(name), RT_RCDATA);
		if (!rsrcData)
			throw std::runtime_error("Failed to find resource.");

		DWORD rsrcDataSize = SizeofResource(handle, rsrcData);
		if (rsrcDataSize <= 0) {
			MessageBox(NULL, L"Size of resource is 0.", L"ERROR", MB_OK);
			throw std::runtime_error("Size of resource is 0.");
		}

		HGLOBAL grsrcData = LoadResource(handle, rsrcData);
		if (!grsrcData)
			throw std::runtime_error("Failed to load resource.");

		LPVOID firstByte = LockResource(grsrcData);
		if (!firstByte)
			throw std::runtime_error("Failed to lock resource.");

		sf::Image image;
		if (!image.loadFromMemory(firstByte, rsrcDataSize))
			throw std::runtime_error("Failed to load image from memory.");

		return image;
	}

	sf::Texture& TextureManager::GetTexture(int resourceId)
	{
		m_images[resourceId] = LoadImageFromResource(resourceId);
		auto& texture = m_textures[resourceId];
		texture.loadFromImage(m_images[resourceId]);
		texture.setSmooth(true);
		return texture;
	}
}