//// ld46.cpp : Defines the entry point for the application.
////
//
#include "stdafx.h"
#include "ld46.h"
#include "TextureManager.h"
#include "SoundManager.h"
#include "Enemy.h"
#include "EnemyMgr.h"
#include "Hero.h"
#include "MusicManager.h"
#include "FontTextMgr.h"
#include "InputManager.h"
#include "HookedRenderWindow.h"
#include "Level.h"
#include "Hud.h"
#include "ParticleMgr.h"


#include "resource.h"
#include <SFML/Graphics.hpp>
#include <sstream>
#include <iomanip>
#include <locale>
#include <codecvt>
#include <iostream>

using namespace LD46;
//
#define MAX_LOADSTRING 100
//
//// Global Variables:
HINSTANCE hInst;                                // current instance
WCHAR szTitle[MAX_LOADSTRING];                  // The title bar text
WCHAR szWindowClass[MAX_LOADSTRING];            // the main window class name


namespace
{
}
												//
//// Forward declarations of functions included in this code module:
//ATOM                MyRegisterClass(HINSTANCE hInstance);
//BOOL                InitInstance(HINSTANCE, int);
//LRESULT CALLBACK    WndProc(HWND, UINT, WPARAM, LPARAM);
//INT_PTR CALLBACK    About(HWND, UINT, WPARAM, LPARAM);
//
int APIENTRY wWinMain(_In_ HINSTANCE hInstance,
                     _In_opt_ HINSTANCE hPrevInstance,
                     _In_ LPWSTR    lpCmdLine,
                     _In_ int       nCmdShow)
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);

	hInst = hInstance;
    // Initialize global strings
    LoadStringW(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
    LoadStringW(hInstance, IDC_LD46, szWindowClass, MAX_LOADSTRING);

	MusicManager musicMgr;
	musicMgr.LoadFromResource(IDS_MUSIC);
	musicMgr.LoadFromResource(IDS_MUSIC_INGAME);

	musicMgr.Play(IDS_MUSIC);

	sf::ContextSettings settings;
	settings.antialiasingLevel = 8;
	sf::Vector2u winSize(640, 400);
	if (wcscmp(lpCmdLine, L"double") == 0)
		winSize = sf::Vector2u(1280, 800);

	HookedRenderWindow window(settings, winSize);
	window.setVerticalSyncEnabled(true);
	window.setFramerateLimit(60);

	if (!sf::Shader::isAvailable())
		return -1;
	
	FontTextMgr fontTextMgr;
	
	const int MAXPOWERUPS = 5;

	sf::Shader shd;
	auto vsh = fontTextMgr.LoadTextInResource(IDB_VERTEXSHADER);
	auto psh = fontTextMgr.LoadTextInResource(IDB_PIXELSHADER);
 	shd.loadFromMemory(vsh, psh);
	shd.setParameter("texture", sf::Shader::CurrentTexture);
	shd.setUniform("resolution", sf::Vector2f(640.0f, 480.0f));
	window.SetShader(&shd);
	sf::Shader skyShdr;
	auto skyPsh = fontTextMgr.LoadTextInResource(IDB_SKYPIXELSHADER);
	skyShdr.loadFromMemory(vsh, skyPsh);
	skyShdr.setParameter("texture", sf::Shader::CurrentTexture);
	skyShdr.setUniform("resolution", sf::Vector2f(640.0f, 480.0f));

	InputManager inputMgr(window);
	inputMgr.DetectJoystick();

	TextureManager texMgr;
	SoundManager sndMgr;
	auto isTitleScreen = true;
	int roundCount = 0;
	while (window.isOpen())
	{
		sf::Clock titleTime;

		sf::Sprite titleImg;
		titleImg.setTexture(texMgr.GetTexture(IDB_TITLESCREEN));
		titleImg.setPosition(sf::Vector2f(0.0f, 0.0f));
		auto isGameOver = false;
		//auto baseLife = 1.0;

		{
			while (window.isOpen() && isTitleScreen)
			{
				sf::Event event;
				while (window.pollEvent(event))
				{
					if (event.type == sf::Event::Closed)
						window.close();
					if (event.type == sf::Event::KeyReleased)
					{
						isTitleScreen = false;
						//firstConnectedJoy = -1; //The player decided to use the keyboard
					}
					if (event.type == sf::Event::JoystickButtonReleased)
						isTitleScreen = false;
				}
				window.clear();
				window.draw(titleImg);
				window.display();
			}
		}
		musicMgr.Stop(IDS_MUSIC);

		while (!isGameOver && window.isOpen())
		{
			musicMgr.Play(IDS_MUSIC_INGAME);
			++roundCount;

			sf::Clock inGameTime;
			sf::Time timeSinceLastThrust;
			int levelTotalTime = 90;
			auto hero = std::make_shared<Hero>(texMgr, sndMgr, 1.0);			
			ParticleMgr particleMgr(texMgr, sndMgr);
			Hud hud(texMgr, fontTextMgr, hero);
			EnemyMgr enemyMgr(texMgr, sndMgr, particleMgr,hud, 0);
			Level level(texMgr, sndMgr, fontTextMgr, particleMgr);
			level.SetHero(hero);

			auto softTick = 0.0;
			Action prevAction = Action::None;
			float rotation = 0.0f;
			bool toggleInfo = false;
			if (roundCount < 2)
				hud.TextDisplay("Come closer...", 5000);
			else 
				hud.TextDisplay("Hmmm...\nThe spawn of darkness got you?\nPlease continue.\nI will revive you as part of our deal!", 5000);
			int textInfo2 = 0;
			while (window.isOpen() && !level.IsLevelComplete() && !isGameOver)
			{
				sf::Vector2i posAdjust;

				inputMgr.Update();
				sf::Vector2f walkVector;
				sf::Vector2f walkDir;
				sf::Vector2f plausibleRotation(1.0f*sin(rotation), 1.0f * cos(rotation));
				Action selectedAction = Action::None;
				if (inputMgr.IsAction(InputAction::MainButton_A))
				{
					if (hero->GetWorldState() != WorldState::Shadow)
					{
						selectedAction = Action::Attack;
						level.HeroShoot();
					}
				}
				if (inputMgr.IsAction(InputAction::Button_X))
				{
					hero->SetWorldState(WorldState::Chaos);
					//auto ocnt = hero->ObjectCount(ObjectType::WaterSpell);
				}
				if (inputMgr.IsAction(InputAction::Button_Y))
				{
					hero->SetWorldState(WorldState::Shadow);
				}

				if (inputMgr.IsAction(InputAction::BackButton_B))
				{
					hero->SetWorldState(WorldState::Darkness);
					//isGameOver = true;
				}

				if (inputMgr.IsAction(InputAction::LeftButton))
				{
					toggleInfo = !toggleInfo;
					hud.ShowInfoText(toggleInfo);

				}
				if (inputMgr.IsAction(InputAction::Left))
				{
					//walkVector.x = -plausibleRotation.x;
					rotation += 0.02;
				}
				if (inputMgr.IsAction(InputAction::Right))
				{
					//walkVector.x = plausibleRotation.x;
					rotation -= 0.02;
				}
				if (inputMgr.IsAction(InputAction::Up))
				{
					walkVector.x = -plausibleRotation.x;
					walkVector.y = -plausibleRotation.y;
				}
				if (inputMgr.IsAction(InputAction::Down))
				{
					walkVector.x = plausibleRotation.x;
					walkVector.y = plausibleRotation.y;
				}				
				hero->SetRotation(rotation);
				const auto& p = hero->Position();

				const auto& knockers = hero->KnockBack();
				if (fabs(knockers.x) > 0 || fabs(knockers.y) > 0)
				{
					if (level.CanWalk(p.x + knockers.x, p.y + knockers.y))
						hero->KnockBackAdjust(knockers);
					else if (level.CanWalk(p.x + knockers.x, p.y))
						hero->KnockBackAdjust(sf::Vector2f(knockers.x, 0.0));
					else if (level.CanWalk(p.x, p.y + knockers.y))
						hero->KnockBackAdjust(sf::Vector2f(0.0, knockers.y));
				}

				if (selectedAction == Action::Attack)
				{
					hero->Act(selectedAction);
				}
				else
				{
					if (level.CanWalk(p.x + walkVector.x, p.y + walkVector.y))
						hero->Walk(walkVector);
					else if (level.CanWalk(p.x + walkVector.x, p.y))
						hero->Walk(sf::Vector2f(walkVector.x, 0.0));
					else if (level.CanWalk(p.x, p.y + walkVector.y))
						hero->Walk(sf::Vector2f(0.0, walkVector.y));
				}


				auto tm = inGameTime.getElapsedTime();
				prevAction = selectedAction;
				if (roundCount < 2)
				{
					if (textInfo2 < 6)
					{
						auto& mm = enemyMgr.MetMystic();
						if (mm.has_value() && (tm - mm.value()).asMilliseconds() > 5000 * textInfo2)
						{
							switch (textInfo2)
							{
							case 0:
								hud.TextDisplay("The tusk you've been gifted, will allow you travel\nbetween realms: Darkness, Shadow & Chaos.", 5000);
								break;
							case 1:
								hud.TextDisplay("Darkness\nThe reality you know.\nPiercing fiends with my tusk wil banish\nthem from all known realms.", 5000);
								break;
							case 2:
								hud.TextDisplay("Shadow\nYour weapons are of no use here,\nbut the shadow realm sometimes offers an\nalternate route when things seem impossible.", 7000);
								break;
							case 3:
								hud.TextDisplay("Chaos.\nYou can't stay long in the realm of chaos,\nit will kill you. But it opens up a path of stronger magic abilities.\nUse them wiseley", 8000);
								break;
							case 4:
								hud.TextDisplay("The triangle imprinted on you will\nshow you your life span in all realms.\n", 5000);
								break;
							case 5:
								hud.TextDisplay("Now for instance might be a good time\nto visit the shadow realm to\nfind your way out of this fleeting sanctuary.", 7000);
								break;
							}
							++textInfo2;
						}
					}
				}

				hud.Update(tm);
				particleMgr.Update(tm, *hero, level);
				
				const auto& shotIds = level.GetHeroShots();
				std::vector<int> collidedShots;
				for (auto s : shotIds)
				{
					if (particleMgr.Particle(s) == nullptr)
						collidedShots.push_back(s);
					else
					{
						const auto& pos = particleMgr.ParticlePos(s);
						if (enemyMgr.UpdateShotCollision(pos))
							collidedShots.push_back(s);
					}
				}
				level.RemoveHeroShots(collidedShots);

				enemyMgr.Update(tm, *hero, level);
				hero->Update(tm);

				softTick += 0.1;
				level.Update(tm);

				//window.clear();
				level.Draw(window, shd, skyShdr);
				hud.Draw(window);
				window.display();

				for (auto i = 0; i != WorldState::COUNT; ++i)
				{
					if (hero->Power(static_cast<WorldState>(i)) <= 0.0)
						isGameOver = true;
				}
			}

			if (!isGameOver)
			{
				//sf::Sprite youWinTxtSpr;
				//youWinTxtSpr.setTexture(texMgr.GetTexture(IDB_YOUMADEIT));
				//youWinTxtSpr.setPosition(160.0f, 100.0f);
				//youWinTxtSpr.setOrigin(160, 100);

				//float scale = 0.0;

				//while (window.isOpen() && scale < 3.14)
				//{
				//	sf::Event event;
				//	while (window.pollEvent(event))
				//	{
				//		if (event.type == sf::Event::Closed)
				//			window.close();
				//	}

				//	scale += 0.01f;
				//	auto s = sin(scale);
				//	youWinTxtSpr.setScale(s, s);
				//	youWinTxtSpr.setRotation(s * 360);
				//	window.draw(youWinTxtSpr);
				//	window.display();
				//}
			}
			else
			{
				//sf::Sprite gameOverTxtSpr;
				//gameOverTxtSpr.setTexture(texMgr.GetTexture(IDB_DEATHTXT));
				//gameOverTxtSpr.setPosition(160.0f, 100.0f);
				//gameOverTxtSpr.setOrigin(160, 100);

				//float scale = 0.0;

				//while (window.isOpen() && scale < 3.14)
				//{
				//	sf::Event event;
				//	while (window.pollEvent(event))
				//	{
				//		if (event.type == sf::Event::Closed)
				//			window.close();
				//	}

				//	scale += 0.01f;
				//	auto s = sin(scale) * 2;
				//	gameOverTxtSpr.setScale(s, s);
				//	window.draw(gameOverTxtSpr);
				//	window.display();
				//}
			}
		}
		musicMgr.Stop(IDS_MUSIC_INGAME);
	}
	

	return 0;
}
